//////////////////////////////////////////////////////////////////////////
//
//        File  Name : win.cc
//        Class Name : window
//        Author     : Young-ki Hwang
//        Date       : April 30, 1996
//        Description: This is methods of "window" class. 
//                     
//
//////////////////////////////////////////////////////////////////////////

#include <curses.h>
#include <stdio.h>
#include "win.h"
 

// From the C library functions.
extern "C" newin(int, int, int, int);
extern "C" wclear(WINDOW *);
extern "C" wclrtoeol(WINDOW *);
extern "C" box(WINDOW *, int, int);
extern "C" wgetch(WINDOW *);
extern "C" wprintw(WINDOW *, ...);
extern "C" waddch(WINDOW *, int);
extern "C" wdelch(WINDOW *);
extern "C" wrefresh(WINDOW *);
extern "C" wmove(WINDOW *, int, int);


//
//  constructor: window()
//  Make a new window which has "line" rows and "col" columns. The start
//  pointer of the window is (begin_y, begin_x) in coordinate of the screen
//
window::window(int line, int col, int begin_y, int begin_x)
{
   lines = line;
   cols  = col;
   start_line = begin_y;
   start_col = begin_x;
 
   // Make a new window.
   win = newwin(line, col, begin_y, begin_x);
}



//
//  makescroll()
//  Make a window to be scrollable.
//
window::makescroll()
{
   scrollok(win, TRUE);
} 
  


//
//  clearall()
//  Clear the window.
//
window::clearall()
{  int i;
  
   for(i=0; i<lines; i++)
     { moveto(i, 0);
       cleartoeol();
     }
}


//
//  cleartoeol()
//  Wipes the window clear from the (y, x) coordinate to the end of the line
//
window::cleartoeol()
{
   wclrtoeol(win);
   display();
}


//
//  cleartobottom()
//  Wipes the window clear from the (y, x) coordinate to the bottom
//
window::cleartobottom()
{
   wclrtobot(win);
   display();
}


//
//  drawborder()
//  Draw a border around the window with vertical character and horizontal
//  character which is from the arguments.
//
window::drawborder(char vert , char hor)
{
   box(win, vert, hor);
   mvwaddch(win, start_line, start_col, '+');
   mvwaddch(win, start_line+lines-1, start_col, '+');
   mvwaddch(win, start_line, start_col+cols-1, '+');
   mvwaddch(win, start_line+lines-1, start_col+cols-1, '+');
   display();
}


//
//  getstring()
//  Get a character from the window and put the character into the string.
//  
window::getstring(char *str)
{
   char  ch,  *temp;

   temp = str;
 
   for( ; ; )
    { ch=getcharacter();
      putcharacter(ch);
      switch(ch)
	{ case '\r': *str = '\0';
		     return;
	  case '\b': if(str > temp)
		     { str--; 
		       get_cur_coordinate();
                       moveto(currenty, currentx);
                       cleartoeol();
		     }
                     break;
	  default  : *str++ =  ch;
	  }
        }
}


//
//  getcharacter()
//  Get a character from the window.
//
char window::getcharacter()
{
   return(  wgetch(win) );
}


//
//  putstring()
//  Write a string onto the window.
// 
window::putstring(char *str)
{
   wprintw(win, "%s\n", str);
   display();
}


//
//  putcharacter()
//  Write a character onto the window.
//
window::putcharacter(char ch)
{
   waddch(win, ch);
   display();
}


//
//  deletecharacter()
//  Delete a character from the window.
//
window::deletecharacter()
{
   wdelch(win);
   display();
}


//
//  display()
//  Repaint the window.
//
window::display()
{
   wrefresh(win);
}


//
//  destroy()
//  Delete the window.
//
window::destroy()
{
   delwin(win);
}


//
//  moveto()
//  Move the cursor to the position (y, x).
//
window::moveto(int y, int x)
{
   wmove(win, y , x);
   display();
}


//
//  get_cur_coordinate()
//  Get the current position of cursor.
//
window::get_cur_coordinate()
{ 
   int y, x;

   getyx(win, y, x);
   currenty = y;
   currentx = x;
}



